/*
 * Copyright (C) 2000 Arne Schirmacher <arne@schirmacher.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gnome.h>
#include <iostream>
#include <vector>


using std::cout;
using std::endl;
using std::vector;

#include "previewer.h"
#include "commands.h"

extern "C" {
#include "support.h"
}


Previewer::Previewer()
{
    gc = NULL;
}


Previewer::~Previewer()
{
}


void Previewer::init(title_sequence seq)
{
   sequence = seq;
   sqi = sequence.begin();
}


void Previewer::draw()
{
   int w, h;
   int fgc, bgc;
   
   if (sqi==sequence.end()) {
      cout << "Out of seqs " << endl;
      return;
   }
   
     
   GtkWidget *drawingarea = lookup_widget( app, "previewArea" );
   Page *page = &(*sqi);
   
   if (gc == NULL) {
      gc = gdk_gc_new(drawingarea->window);
      gdk_gc_get_values(gc, &values);
   }
   bgc = (page->is_opaque) ? ((page->bgcolor.r<<16) + (page->bgcolor.g<<8) + page->bgcolor.b):
                              0xFF000000;   
   fgc = (page->fgcolor.r<<16) + (page->fgcolor.g<<8) + page->fgcolor.b;
   
   w = drawingarea->allocation.width;
   h = drawingarea->allocation.height;
   
   // Background
   gdk_rgb_gc_set_foreground(gc,bgc);
   gdk_draw_rectangle(drawingarea->window, gc,
		      true, 0, 0, w, h); 
	
   // Text
   GdkFont* font = gdk_font_load (page->font.c_str());
   // Fix the size for the display
   
   gdk_rgb_gc_set_foreground(gc,fgc);
//   gdk_rgb_gc_set_foreground(gc,0x00FFFFFF);
   gdk_gc_set_font(gc, font);
//   gdk_draw_rectangle(drawingarea->window, gc, true, 0, 0, 20, 10);
   gdk_draw_text(drawingarea->window, font, gc, 20, 20, page->text.c_str(), page->text.length());
}

void Previewer::first()
{   
   sqi = sequence.begin();
}

void Previewer::last()
{
   sqi = sequence.end();
   sqi--;
}

void Previewer::step_i()
{
   if (sqi!=sequence.begin())
     sqi--;
}

void Previewer::step()
{
   if (sqi!=sequence.end())
     sqi++;
}


