/*
 * Copyright (C) 2001 Alejandro Aguilar Sierra <asierra@servidor.unam.mx>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <iostream>
using std::cout;
using std::endl;

#include "titler_common.h"
#include "commands.h"
#include "previewer.h"

extern "C" 
{   
   #include "interface.h"
   #include "support.h"
   
   #ifdef MULTITHREADED
   #include <pthread.h>
   
   #endif
}


Previewer preview;

GtkWidget *gnomeDialog = NULL;

std::ostream& operator<<(std::ostream& o, const Color& c)
{   
   return o << int(c.r) << ',' << int(c.g) << ',' << int(c.b) << ':' << int(c.a);
}

std::ostream& operator<<(std::ostream& o, const Page& p)
{
   return o << "Text    :" << p.text << endl
     << "Font    :" << p.font << endl
     << "Opaque  :" << p.is_opaque << endl
     << "Duration:" << p.duration << endl
     << "Color   :" << p.fgcolor << endl
     << "Position:" << p.position << endl;
}


Page::Page()
{
//   text = new string();
}

Page::~Page()
{
//   delete text;
}


TitlerCommon::TitlerCommon()
{
}

TitlerCommon::~TitlerCommon()
{
}

void TitlerCommon::insertPage() 
{
   gnomeDialog = create_dialog_page ();
   resetDialogPage();
   gtk_widget_show (gnomeDialog);
}


void TitlerCommon::resetDialogPage()
{   GtkWidget *opmenu;
   GtkWidget *menu;
   GtkWidget *item;

   // Initial position menus
   opmenu = lookup_widget(gnomeDialog, "optionHor");
   gtk_option_menu_set_history(GTK_OPTION_MENU (opmenu), 0);
   
   opmenu = lookup_widget(gnomeDialog, "optionVer");
   gtk_option_menu_set_history(GTK_OPTION_MENU (opmenu), 0);
   
   // Initial transition values
   opmenu = lookup_widget(gnomeDialog, "optionTrin");  
   gtk_option_menu_set_history(GTK_OPTION_MENU (opmenu), 0);
   
   opmenu = lookup_widget(gnomeDialog, "optionTrout");  
   gtk_option_menu_set_history(GTK_OPTION_MENU (opmenu), 0);
   
   // Other default values
   Color c;
   c.r = c.g = c.b = c.a = 255;
   item = lookup_widget(gnomeDialog, "colorfgpicker");
   gnome_color_picker_set_i8 (GNOME_COLOR_PICKER(item), c.r, c.g, c.b, c.a);
   
   item = lookup_widget(gnomeDialog, "colorbgpicker");   
   c.r = c.g = c.b = 0; c.a = 255;
   gnome_color_picker_set_i8 (GNOME_COLOR_PICKER(item), c.r, c.g, c.b, c.a);

   // Text
   item = lookup_widget(gnomeDialog, "newtext");
   int l = gtk_text_get_length(GTK_TEXT(item));
   gtk_editable_delete_text(GTK_EDITABLE(item), 0, l); 
}

void TitlerCommon::updateDialogPage()
{
   GtkWidget *opmenu;
   GtkWidget *menu;
   GtkWidget *item;
   GtkWidget *widget;

   // Text
   item = lookup_widget(gnomeDialog, "newtext");
   int l = gtk_text_get_length(GTK_TEXT(item));
   gtk_editable_delete_text(GTK_EDITABLE(item), 0, l);
   gtk_editable_insert_text(GTK_EDITABLE(item), current_page.text.c_str(),
			    current_page.text.length(),
			    &l);
   
   // Init position menus
   opmenu = lookup_widget(gnomeDialog, "optionHor");
   gtk_option_menu_set_history(GTK_OPTION_MENU (opmenu), current_page.position & 0x03);
   
   opmenu = lookup_widget(gnomeDialog, "optionVer");
   gtk_option_menu_set_history(GTK_OPTION_MENU (opmenu), 
			       (current_page.position>>2) & 0x03);
      
   // Other default values
   Color c = current_page.fgcolor;
   item = lookup_widget(gnomeDialog, "colorfgpicker");
   gnome_color_picker_set_i8 (GNOME_COLOR_PICKER(item), c.r, c.g, c.b, c.a);
   
   item = lookup_widget(gnomeDialog, "colorbgpicker");
   c = current_page.bgcolor;
   gnome_color_picker_set_i8 (GNOME_COLOR_PICKER(item), c.r, c.g, c.b, c.a);
   
   widget = lookup_widget(gnomeDialog, "fontpicker");
   gnome_font_picker_set_font_name(GNOME_FONT_PICKER(widget), current_page.font.c_str());
}


void TitlerCommon::okPage()
{
   GtkWidget *widget;
   GtkWidget *dialog;
   GtkWidget *item;
   char *s;
   
   dialog = gnomeDialog;
   
   // Text
   widget = lookup_widget(dialog, "newtext");
   current_page.text = gtk_editable_get_chars(GTK_EDITABLE(widget),0,-1);
   
   widget = lookup_widget(dialog, "fontpicker");
   current_page.font = gnome_font_picker_get_font_name(GNOME_FONT_PICKER(widget));
   
   widget = lookup_widget(dialog, "colorfgpicker");
   Color *c = &current_page.fgcolor; 
   gnome_color_picker_get_i8(GNOME_COLOR_PICKER(widget), &c->r, &c->g, &c->b, &c->a);
   
   // Background
   widget = lookup_widget(dialog, "checkOpaque");
   current_page.is_opaque = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   
   if (current_page.is_opaque) {	
      widget = lookup_widget(dialog, "colorbgpicker");
      Color *c = &current_page.bgcolor;
      gnome_color_picker_get_i8(GNOME_COLOR_PICKER(widget), &c->r, &c->g, &c->b, &c->a);
   }
   
   // Timing
   widget = lookup_widget(dialog, "entryTime");
   s = gtk_editable_get_chars(GTK_EDITABLE(widget),0,-1);
   current_page.duration = atoi(s);
}


void TitlerCommon::feedClist()
{   
   GtkWidget *widget;
   char *labels[3];
   title_sequence::iterator lci;
   
   widget = lookup_widget(app, "clist");
   
   for(lci=sequence.begin(); lci!=sequence.end(); lci++) {
      labels[0] = "TC Info";
      (const char*)labels[1] = lci->text.c_str();      
      labels[2] = "Attributes";      
      gtk_clist_append(GTK_CLIST(widget), labels);
   }
}

void TitlerCommon::appendClist()
{   
   GtkWidget *widget;
   char *labels[3];
   title_sequence::iterator lci;;
   
   fprintf(stdout, "App %x\n", app);
   fflush(stdout);
   
   widget = lookup_widget(app, "clist");
      
   labels[0] = "hola";
   (const char*)labels[1] = current_page.text.c_str();
   labels[2] = "cola";
   gtk_clist_append(GTK_CLIST(widget), labels);
}

void TitlerCommon::vcr(vcr_type command)
{	
   cout << "VCR " << command << endl;

   switch(command) 
     {
      case VCR_PLAY:
	preview.init(sequence);
//	preview.play();
	break;
      case VCR_STEP:
	preview.step();
	break;
      case VCR_STEP_I:
	preview.step_i();
	break;
      case VCR_FIRST:
	preview.first();
	break;
      case VCR_LAST:
	preview.last();
	break;
      case VCR_STOP:
//	feedClist();
	break;
	
     }	
   preview.draw();
}

void TitlerCommon::setPosition(int pos) 
{
   // Mask it
   current_page.position &= (pos < 3) ? 0x03: 0x0c;
   
   current_page.position |= pos;
}

void TitlerCommon::pageClicked(int data)
{
   switch(data) 
     {
      case 0:
	gnome_dialog_close(GNOME_DIALOG(gnomeDialog));
	gnomeDialog = NULL;
	break;
      case 1:
	okPage();
	sequence.push_back(current_page);
	appendClist();	
	resetDialogPage();
	cout << current_page;
	break;
      case 2:      
	okPage();
//	refreshClist();
	cout << current_page;
	break;
     }   
}

void TitlerCommon::pageSelected(int row)
{
   title_sequence::iterator lci = sequence.begin();
   
   while (row > 0 && lci!=sequence.end()) {
      lci++;
      row--;
   }
   current_page = *lci;
   if (gnomeDialog==NULL) {	
      gnomeDialog = create_dialog_page ();
      updateDialogPage();
      gtk_widget_show (gnomeDialog);
   } else
     updateDialogPage();
}

void TitlerCommon::effectSelected(int data) 
{
   if (gnomeDialog==NULL)
     return;

   GtkWidget *trin = lookup_widget(gnomeDialog, "optionTrin");
   GtkWidget *dirin = lookup_widget(gnomeDialog, "optiondirIn");
   GtkWidget *trout = lookup_widget(gnomeDialog, "optionTrout");
   GtkWidget *dirout = lookup_widget(gnomeDialog, "optiondirOut");
   GtkWidget *scrollb = lookup_widget(gnomeDialog, "buttoneff_scroll");
   GtkWidget *crawlb = lookup_widget(gnomeDialog, "buttoneff_crawl");
   
   gtk_option_menu_set_history(GTK_OPTION_MENU (trin), 1);
   gtk_option_menu_set_history(GTK_OPTION_MENU (trout), 1);
   switch(data) 
     {
      case 0:
	gtk_option_menu_set_history(GTK_OPTION_MENU (dirin), 3);
	gtk_option_menu_set_history(GTK_OPTION_MENU (dirout), 3);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(crawlb), FALSE);
	break;
      case 1:
	gtk_option_menu_set_history(GTK_OPTION_MENU (dirin), 1);
	gtk_option_menu_set_history(GTK_OPTION_MENU (dirout), 1);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(scrollb), FALSE);
	break;
     }   
}
