/*
 * Copyright (C) 2001 Alejandro Aguilar Sierra <asierra@servidor.unam.mx>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _TITLER_COMMON_H
#define _TITLER_COMMON_H 1

#include <iostream>

#include <string>
using std::string;

#include <vector>
using std::vector;

#include <list>
using std::list;

#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gnome.h>

#include "commands.h"

typedef enum transition_type 
{   
     TRANSITION_CUT,
     TRANSITION_FADE,
     TRANSITION_SCROLLUP,
     TRANSITION_SCROLLDOWN,
     TRANSITION_SCROLLLEFT,
     TRANSITION_SCROLLRIGHT,
     TRANSITION_UNDEFINED
};


typedef enum position_type
{ 
     POSITION_LEFT=1,
     POSITION_RIGHT=2,
     POSITION_CENTER=3,
     POSITION_BOTTOM=4,
     POSITION_TOP=8,
     POSITION_MIDDLE=12,
     POSITION_UNDEFINED
};


/// Currently this class is very simple but will grow later
typedef struct Transition {
   /// Number of frames this transition lasts
   int duration;     
   transition_type type;
} Transition;


typedef string Font;

class Color 
{
 public:
   guint8 r, g, b, a;
};

std::ostream& operator<<(std::ostream& o, const Color& c);


class Page
{   
 public:
   Page();
   ~Page();

   string text;
   bool is_opaque;
   Color fgcolor;
   Color bgcolor;
   Font font;
   int duration;

   int position;
   Transition tr_in;
   Transition tr_out;
};

std::ostream& operator<<(std::ostream& o, const Page& p);

typedef list<Page> title_sequence;

class TitlerCommon 
{
 public:
   TitlerCommon();
   ~TitlerCommon();
   /// New page
   void insertPage();
   /// Update existing page
   void updatePage();
   void okPage();
   void appendClist();
   void feedClist();
   void vcr(vcr_type);
   void setPosition(int);
   void pageClicked(int);
   void pageSelected(int);
   void effectSelected(int data);
   
 protected:
   void initDialogPage();
   void updateDialogPage();
   void resetDialogPage();
   
 private:
   title_sequence sequence;
   Page current_page;
};


#endif

